/**
 * Application configuration
 */

import moment from 'moment';
import { PresetManager } from '@bryntum/schedulerpro';

const zoomLevels = {
    minuteAndHour: 1,
    hourAndDay: 1
};

const schedulerConfig = {
    // Limit zoom levels to those which use hours to make filtering non-working time works better in this demo
    presets: PresetManager.records.filter((preset) => zoomLevels[preset.id]),

    // A Project holds the data and the calculation engine for Scheduler Pro. It also acts as a CrudManager, allowing
    // loading data into all stores at once
    project: {
        autoLoad: true,
        transport: {
            load: {
                url: './data/data.json'
            }
        }
    },

    startDate: '2023-05-04',
    endDate: '2023-05-06',
    rowHeight: 90,
    barMargin: 15,
    eventStyle: 'border',
    resourceImagePath: './users/',

    // Custom viewPreset (based on 'hourAndDay') that displays a compact 24 hour bottom header
    viewPreset: {
        base: 'hourAndDay',
        tickWidth: 30,
        displayDateFormat: 'll HH:mm',
        headers: [
            {
                unit: 'day',
                dateFormat: 'ddd DD/MM' //Mon 01/10
            },
            {
                unit: 'hour',
                dateFormat: 'HH'
            }
        ]
    },

    // Not using the dependencies feature
    dependenciesFeature: false,

    nestedEventsFeature: {
        constrainDragToParent: true
    },
    eventNonWorkingTimeFeature: {
        disabled: true
    },
    eventTooltipFeature: {
        template: (data) => {
            let customersNotification = '';
            if (
                data.eventRecord.customers &&
                data.eventRecord.customers.length > 0
            ) {
                for (const i of data.eventRecord.customers) {
                    if (customersNotification !== '') {
                        customersNotification += ' | ';
                    }
                    customersNotification += i.name;
                }
            }
            const htmlReturn = `
            <span style="width: 350px; display: flex; flex-direction: column">
              <span style="font-weight: bold">
                ${
                    data.eventRecord.children
                        ? `${data.eventRecord.data.vehicle} (${data.eventRecord.data.carrierCode}) - ${data.eventRecord.data.type} | ${data.eventRecord.data.number}`
                        : data.eventRecord.data.childType === 'U'
                        ? 'unloading'
                        : 'loading'
                }
              </span>
              ${
                  data.eventRecord.children
                      ? `<span style="font-size: small">${customersNotification}</span>`
                      : `<span style="font-size: small">${data.eventRecord.data.locationDescription}: ${data.eventRecord.data.type} | ${data.eventRecord.data.number}</span>`
              }
              <br />
              ${
                  data.eventRecord.data.childType === 'U' ||
                  data.eventRecord.data.childType === 'L'
                      ? `<span style="display: flex">
                  ${data.startClockHtml}&nbsp;${moment(
                            data.eventRecord.data.startDate
                        ).format('HH:mm')}&nbsp;-&nbsp;${moment(
                            data.eventRecord.data.endDate
                        ).format('HH:mm')}
                    </span>`
                      : data.eventRecord.data.modalityType === 'ship'
                      ? `<span style="display: flex">
                  ${data.startClockHtml}&nbsp;-&nbsp;${moment(
                            data.eventRecord.data.startDate
                        ).format('HH:mm')}
                    </span>`
                      : `<span style="display: flex">
                    ${data.startClockHtml}&nbsp;${moment(
                            data.eventRecord.data.startDate
                        ).format('HH:mm')} - ${moment(
                            data.eventRecord.data.endDate
                        ).format('HH:mm')}
                      </span>`
              }
              ${
                  data.eventRecord.data.childType === 'U' ||
                  data.eventRecord.data.childType === 'L' ||
                  data.eventRecord.data.modalityType === 'truck'
                      ? ''
                      : `<span style="display: flex">
                  ${data.endClockHtml}&nbsp;-&nbsp;${moment(
                            data.endDate
                        ).format('HH:mm')}
                </span>`
              }
              </span>
            ${
                data.eventRecord.data.children
                    ? `<br />
               <span style="width: 350px; display: flex;">
                <span style="width: 50%; font-size: small; font-weight: bold;">
                  loading
                </span>
                <span style="width: 50%; font-size: small; font-weight: bold;">
                  unloading
                </span>
              </span>
              <span style="width: 350px; display: flex;">
                <span style="width: 50%; font-size: small;">
                ${data.eventRecord.data.children
                    .filter((x) => x.childType === 'L')
                    .sort((leftSubEvent, rightSubEvent) => {
                        return leftSubEvent.startDate - rightSubEvent.startDate;
                    })
                    .map(
                        (child) =>
                            `<div>${child.locationDescription}: ${moment(
                                child.startDate
                            ).format('HH:mm')} - ${moment(child.endDate).format(
                                'HH:mm'
                            )}</div>`
                    )}
                </span>
                <span style="width: 50%; font-size: small;">
                ${data.eventRecord.data.children
                    .filter((x) => x.childType === 'U')
                    .sort((leftSubEvent, rightSubEvent) => {
                        return leftSubEvent.startDate - rightSubEvent.startDate;
                    })
                    .map(
                        (child) =>
                            `<div>${child.locationDescription}: ${moment(
                                child.startDate
                            ).format('HH:mm')} - ${moment(child.endDate).format(
                                'HH:mm'
                            )}</div>`
                    )}
                </span>
              </span>`
                    : ''
            }`;
            return htmlReturn;
        }
    },
    columns: [
        {
            type: 'resourceInfo',
            width: 350,
            showEventCount: false,
            htmlEncode: false,
            filterable: {
                filterField: {
                    triggers: {
                        search: {
                            cls: 'b-icon b-fa-filter'
                        }
                    },
                    placeholder: 'Filter vehicles'
                }
            },
            renderer: ({ record }) => {
                const numberTrips = record.events.filter(
                    (x) => x.children
                ).length;
                let htmlReturn = '';
                const iconClass =
                    record.type === 'ship'
                        ? 'b-fa b-fa-ship'
                        : 'b-fa b-fa-truck';
                const backgroundColor =
                    record.type === 'ship' ? 'blue' : 'green';
                htmlReturn = `
                <div class="b-resource-avatar b-resource-initials" role="presentation" style="background: ${backgroundColor}"><i class="${iconClass}" role="presentation"></i></div>
                <div style="width: 10px"></div>
                <span style="width: 100%; display: flex; flex-direction: column; align-items: start;">
                  ${
                      numberTrips === 0
                          ? `<div">`
                          : `<div style="font-weight: bold;">`
                  }
                    ${record.name}
                  </div>
                  ${
                      numberTrips > 0
                          ? `<div class="b-grid-cell.b-group-title" style="font-size: small">
                  ${numberTrips} trip(s)
                </div>`
                          : ''
                  }
                </span>`;
                return htmlReturn;
            }
        }
    ],
    eventRenderer({ eventRecord }) {
        if (eventRecord.children) {
            return `${eventRecord._data.type} | ${eventRecord._data.number}`;
        }
        return '';
    },

    tbar: [
        // Add a button to filter out non-working time
        {
            ref: 'filterButton',
            type: 'button',
            text: 'Filter out non-working time',
            toggleable: true,
            icon: 'b-fa-square',
            pressedIcon: 'b-fa-check-square'
        }
    ]
};

export { schedulerConfig };
